﻿namespace HeuristicLab.Algorithms.PushGP.Interpreter {
  using System.Collections.Generic;

  using HeuristicLab.Algorithms.PushGP.Expressions;

  public class Configuration {
    public Configuration() {
      this.IsBooleanStackEnabled = true;
      this.IsIntegerStackEnabled = true;
      this.IsFloatStackEnabled = true;
      this.IsCodeStackEnabled = true;
      this.IsNameStackEnabled = true;

      this.EvalPushLimit = 4096;
      this.MaxPointsInProgram = 128;
      this.MaxPointsInRandomExpression = 1024;

      this.TopLevelPushCode = true;
      this.TopLevelPopCode = false;

      this.RandomSeedMax = 30081;
      this.RandomSeedMin = 0;
      this.RandomSeed = null;

      this.MinRandomInteger = -128;
      this.MaxRandomInteger = 128;
      this.MinRandomFloat = -128D;
      this.MaxRandomFloat = 128D;
      this.NewErcNameProbability = 0.5;
    }

    public bool IsBooleanStackEnabled { get; set; }

    public bool IsIntegerStackEnabled { get; set; }

    public bool IsFloatStackEnabled { get; set; }

    public bool IsCodeStackEnabled { get; set; }

    public bool IsNameStackEnabled { get; set; }

    public IList<string> AllowedInstructions { get; set; }

    public IList<Expression> PredefinedExpressions { get; set; }

    /// <summary>
    ///     This is the maximum allowed number of "executions" in a single top-level call to the interpreter.
    ///     The execution of a single Push instruction counts as one execution, as does the processing of a single literal,
    ///     as does the descent into one layer of parentheses (that is, the processing of the "(" counts as one execution).
    ///     When this limit is exceeded the interpreter aborts immediately, leaving its stacks in the states they were in prior
    ///     to the abort (so they may still be examined by a calling program). Whether or not this counts as an "abnormal"
    ///     termination
    ///     is up to the calling program.
    /// </summary>
    public int EvalPushLimit { get; set; }

    /// <summary>
    /// This is the maximum of depth a push program can have. Expressions, which lead to exceed this limit are interpreted as NOOP.
    /// </summary>
    public int MaxDepth { get; set; }

    /// <summary>
    ///     This is the maximum size of an item on the CODE stack, expressed as a number of points.
    ///     A point is an instruction, a literal, or a pair of parentheses. Any instruction that would cause this limit to be
    ///     exceeded
    ///     should instead act as a NOOP, leaving all stacks in the states that they were in before the execution of the
    ///     instruction.
    /// </summary>
    public int MaxPointsInProgram { get; set; }

    /// <summary>
    ///     The maximum number of points in an expression produced by the CODE.RAND instruction.
    /// </summary>
    public int MaxPointsInRandomExpression { get; set; }

    /// <summary>
    ///     When TRUE (which is the default), code passed to the top level of the interpreter
    ///     will be pushed onto the CODE stack prior to execution.
    /// </summary>
    public bool TopLevelPushCode { get; set; }

    /// <summary>
    ///     When TRUE, the CODE stack will be popped at the end of top level calls to the interpreter. The default is FALSE.
    /// </summary>
    public bool TopLevelPopCode { get; set; }

    /// <summary>
    ///     The minimum INTEGER that will be produced as an ephemeral random INTEGER constant or from a call to INTEGER.RAND.
    /// </summary>
    public int MinRandomInteger { get; set; }

    /// <summary>
    ///     The maximum INTEGER that will be produced as an ephemeral random INTEGER constant or from a call to INTEGER.RAND.
    /// </summary>
    public int MaxRandomInteger { get; set; }

    /// <summary>
    ///     The minimum FLOAT that will be produced as an ephemeral random FLOAT constant or from a call to FLOAT.RAND.
    /// </summary>
    public double MinRandomFloat { get; set; }

    /// <summary>
    ///     The maximum FLOAT that will be produced as an ephemeral random FLOAT constant or from a call to FLOAT.RAND.
    /// </summary>
    public double MaxRandomFloat { get; set; }

    /// <summary>
    ///     The probability that the selection of the ephemeral
    ///     random NAME constant for inclusion in randomly generated code will produce a new name
    ///     (rather than a name that was previously generated).
    /// </summary>
    public double NewErcNameProbability { get; set; }

    public ushort RandomSeedMax { get; set; }

    public ushort RandomSeedMin { get; set; }

    public ushort? RandomSeed { get; set; }
  }
}